<?php
/*
 * Copyright(c) 2011 SOFTBANK PAYMENT SERVICE Corp. All rights reserved.
 * http://www.sbpayment.jp/
 */

// {{{ requires
require_once(CLASS_EX_REALDIR . "page_extends/admin/LC_Page_Admin_Ex.php");
require_once(MDL_SPS_CLASS_PATH . "LC_Mdl_SPS.php");

/**
 * 支払いmemo仕様
 * ・dtb_payment
 * memo01 マーチャントID
 * memo02 サービスID
 * memo03 支払い方法(クレジット1, コンビニ2, ペイジー3, ウェブマネー4, Yahoo!ウォレット5)
 * memo04 SPS顧客ID
 * memo05 SPS支払方法管理番号
 * memo06 ハッシュ値
 * memo07
 * memo08
 * memo09
 * memo10
 */

/**
 * SPS決済モジュールの管理画面設定のページクラス.
 *
 * @package Page
 * @author SOFTBANK PAYMENT SERVICE Corp.
 */
class LC_Page_Mdl_SPS_Config extends LC_Page_Admin_Ex {

	// }}}
	// {{{ functions

    /**
	 * Page を初期化する.
     *
     * @return void
     */
    function init() {
        parent::init();
        $objSPS =& LC_Mdl_SPS::getInstance();
        $this->tpl_mainpage = MDL_SPS_TEMPLATE_PATH. 'config.tpl';
        $this->tpl_subtitle = $objSPS->getName();

        $this->arrSettlement = array(SETTLEMENT_LINK => 'リンク型', SETTLEMENT_API => 'API型');
        $this->arrOptionActive = array('1' => '要', '0' => '不要');
        $this->type_sps_auth = array('売上日指定要求', '自動売上');
    }

    /**
     * Page のプロセス.
     *
     * @return void
     */
    function process() {
		$this->action();
		$this->sendResponse();
	}

    /**
     * Page のアクション.
     *
     * @return void
     */
    function action() {
        $objSPS =& LC_Mdl_SPS::getInstance();
        $objSPS->install();
		switch($this->getMode()) {
        case 'register':
            list($this->arrForm, $this->arrErr, $this->tpl_onload) = $this->registerMode($_POST);
            break;
        case 'installplugin':
            $this->lfRegistPlugin(MDL_SPS_CODE);
            list($this->arrForm, $this->tpl_onload) = $this->defaultMode();
            $this->tpl_onload .=  'alert("プラグインをインストールしました。");';
            break;
        default:
            list($this->arrForm, $this->tpl_onload) = $this->defaultMode();
            break;
        }
        $this->tpl_is_module_regist = $this->lfIsRegistPaymentModule();
		$this->setTemplate($this->tpl_mainpage);
    }

    /**
     * デストラクタ.
     *
     * @return void
     */
    function destroy() {
        parent::destroy();
    }

    /**
     * 初回表示処理
     *
     */
    function defaultMode() {
        $objSPS =& LC_Mdl_SPS::getInstance();
        $subData = $objSPS->getUserSettings();
        $tpl_onload = $this->getBodyOnload($subData);
        $objForm = $this->initParam($subData);
        return array($objForm->getFormParamList(), $tpl_onload);
    }

    /**
     * 登録ボタン押下時の処理
     *
     */
    function registerMode(&$arrParam) {
        // 認証情報
        $objSess = new SC_Session_Ex();

        $objSPS =& LC_Mdl_SPS::getInstance();
        $tpl_onload = $this->getBodyOnload($arrParam);
        $objForm = $this->initParam($arrParam);
        if ($arrErr = $this->checkError($objForm)) {
            return array($objForm->getFormParamList(), $arrErr, $tpl_onload);
        }
        $arrForm = $objForm->getHashArray();

        $objSPS->updateFile();
        $arrFailedFile = $objSPS->getFailedCopyFile();
        if (count($arrFailedFile) > 0) {
            foreach($arrFailedFile as $file) {
                $alert = $file . 'に書込権限を与えてください。';
                $tpl_onload .= 'alert("' . $alert . '");';
            }
            return array($objForm->getFormParamList(), $arrErr, $tpl_onload);
        }
        $objSPS->registerUserSettings($arrForm);

        // del_flgを削除にしておく
		$objQuery =& SC_Query::getSingletonInstance();
		$objQuery->begin();
		$arrUpdVal = array('del_flg' => 1);
		$where = 'module_code = ?';
		$arrWhereVal = array(MDL_SPS_CODE);
		$objQuery->update('dtb_payment', $arrUpdVal, $where, $arrWhereVal);

        // 決済登録
        for ($i = 1; $i <= 6; $i++) {
            $arrData = array();
            switch($i) {
                    // クレジット
                    case SPS_CREDIT:
                        $arrData["payment_method"] = "SBPS クレジット";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_cr_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_card.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_cr"];
                        }
                        break;
					case SPS_CREDIT3D:
                        $arrData["payment_method"] = "SBPS 3Dセキュアクレジット";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_cr_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_card.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_cr3d"];
                        }
                        break;
                    // ウェブコンビニ
                    case SPS_CONVENI:
                        $arrData["payment_method"] = "SBPS コンビニ";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_wc_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_conveni.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_wc"];
                        }
                        break;
                    // ペイジー
                    case SPS_PAYEASY:
                        $arrData["payment_method"] = "SBPS ペイジー";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_pe_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_payeasy.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_pe"];
                        }
                        break;
                    // ウェブマネー
                    case SPS_WEBMONEY:
                        $arrData["payment_method"] = "SBPS ウェブマネー";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_wm_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_webmoney.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_wm"];
                        }
                        break;
                    // Yahoo!ウォレット
                    case SPS_YWALLET:
                        $arrData["payment_method"] = "SBPS Yahoo!ウォレット";
                        if ($arrForm['settlement_division'] == SETTLEMENT_API) {
                            $arrData['memo03'] = $arrForm["type_sps_payment_yw_api"];
                            $module_path = MODULE_REALDIR . "mdl_sps/api_yahoowallet.php";
                        } else {
                            $arrData['memo03'] = $arrForm["type_sps_payment_yw"];
                        }
                        break;
			}
			//memo03に値が入っていないとモジュール呼び出しされないので注意。
	        if ($arrData['memo03'] > 0 ) {
			    $arrData["memo01"] = $arrForm["id_sps_marchant"]; // マーチャントID
		        $arrData["memo02"] = $arrForm["id_sps_service"]; // サービスID
				$arrData["memo04"] = $arrForm["id_sps_client"]; // SPS顧客ID
	            $arrData["memo05"] = $arrForm["no_sps_paymentmanage"]; // SPS支払方法管理番号
		        $arrData["memo06"] = $arrForm["id_sps_hash"]; // ハッシュ値
			    $arrData["fix"] = 3;
				$arrData["creator_id"] = $objSess->member_id;
	            $arrData["update_date"] = "now()";
		        $arrData["module_code"] = $objSPS->getCode(true);
			    $arrData["module_path"] = (strlen($module_path) > 0) ? $module_path : MODULE_REALDIR . "mdl_sps/card.php";
				$arrData["del_flg"] = "0";
	            $arrPayment = $this->getPaymentDB($i);

		        // 更新データがあれば更新する。
			    if (count($arrPayment) > 0){
				    // データが存在していればUPDATE、無ければINSERT
					$where = "module_code = ? AND memo03 = ?";
					$arrWhereVal = array($objSPS->getCode(true), (string)$i);
			        $objQuery->update('dtb_payment', $arrData, $where, $arrWhereVal);
			    } else {
				    // ランクの最大値を取得する
					$max_rank = $objQuery->get('max(rank)', 'dtb_payment');
	                $arrData["create_date"] = "now()";
		            $arrData["rank"] = $max_rank + 1;
					$arrData['payment_id'] = $objQuery->nextVal('dtb_payment_payment_id');
			        $objQuery->insert("dtb_payment", $arrData);
	            }
		    }
		}
//        $this->lfRegistPlugin(MDL_SPS_CODE);
//      ここではプラグインインストール出来ない
		$objQuery->commit();

        $tpl_onload .= 'alert("登録完了しました。\n基本情報＞支払方法設定より詳細設定をしてください。");window.close();';
        return array($objForm->getFormParamList(), $arrErr, $tpl_onload);
    }

    /**
     * フォームパラメータ初期化
     *
     * @param array $arrData
     * @return object SC_FormParam
     */
    function initParam($arrData = array()) {
        $objForm = new SC_FormParam();

        $objForm->addParam('システム種別', 'settlement_division', INT_LEN, '', array('EXIST_CHECK', 'ALNUM_CHECK'));
        $objForm->addParam('マーチャントID', 'id_sps_marchant', STEXT_LEN, '', array('EXIST_CHECK', 'ALNUM_CHECK'));
        $objForm->addParam('サービスID', 'id_sps_service'	, STEXT_LEN, '', array('EXIST_CHECK', 'ALNUM_CHECK'));
        $objForm->addParam('ハッシュ値', 'id_sps_hash', SMTEXT_LEN, '', array('EXIST_CHECK', 'ALNUM_CHECK', 'MAX_LENGTH_CHECK'));
        //$objForm->addParam('SPS顧客ID', 'id_sps_client', STEXT_LEN, '', array('ALNUM_CHECK'));
        //$objForm->addParam('SPS支払方法管理番号', 'no_sps_paymentmanage', STEXT_LEN, '', array('ALNUM_CHECK'));

        $objForm->addParam('カード情報保持機能', 'stock_card', INT_LEN, '', array("NUM_CHECK", 'MAX_LENGTH_CHECK'), '0');
        $objForm->addParam('売上計上設定', 'type_sps_auth', INT_LEN, '', array('NUM_CHECK', 'MAX_LENGTH_CHECK'), '0');
        $objForm->addParam('接続先URL', 'url_sps_api', MTEXT_LEN, '', array('EXIST_CHECK', 'URL_CHECK'));

        // システム種別で分岐
        if ($arrData['settlement_division'] == SETTLEMENT_API) {
            $objForm->addParam('3DES暗号化キー', 'key_sps_3des', 24, '', array('EXIST_CHECK', 'ALNUM_CHECK', 'MAX_LENGTH_CHECK'));
            $objForm->addParam('3DES初期化キー', 'key_sps_3des_iv', 8, '', array('EXIST_CHECK', 'ALNUM_CHECK', 'MAX_LENGTH_CHECK'));
            $objForm->addParam('クレジット決済', 'type_sps_payment_cr_api', STEXT_LEN, '', array('ALNUM_CHECK'));
        } else {
            $objForm->addParam('購入要求接続先URL', 'url_sps_buyservice_api', MTEXT_LEN, '', array('EXIST_CHECK', 'URL_CHECK'));
            $objForm->addParam('クレジット決済', 'type_sps_payment_cr', STEXT_LEN, '', array('ALNUM_CHECK'));
            $objForm->addParam('ウェブコンビニ決済', 'type_sps_payment_wc', STEXT_LEN, '', array('ALNUM_CHECK'));
            $objForm->addParam('ペイジー決済', 'type_sps_payment_pe', STEXT_LEN, '', array('ALNUM_CHECK'));
            $objForm->addParam('ウェブマネー決済', 'type_sps_payment_wm', STEXT_LEN, '', array('ALNUM_CHECK'));
            $objForm->addParam('Yahoo!ウォレット決済', 'type_sps_payment_yw', STEXT_LEN, '', array('ALNUM_CHECK'));
			$objForm->addParam('３Dセキュアクレジット', 'type_sps_payment_cr3d', STEXT_LEN, '', array('ALNUM_CHECK'));
        }

        $objForm->setParam($arrData);
        $objForm->convParam();
        return $objForm;
    }

    /**
     * 入力パラメータの検証
     *
     * @param SC_FormParam $objForm
     * @return array|null
     */
    function checkError(&$objForm) {
        $arrErr = null;
        $arrErr = $objForm->checkError();
        // PHP拡張モジュールチェック
        if ($objForm->getValue("settlement_division") == SETTLEMENT_API && extension_loaded("mcrypt") === false) {
            $arrErr['err'] .= "※ Mcrypt拡張モジュールをロードできません。PHPの拡張モジュールをインストールしてください。<br />";
        }
        return $arrErr;
    }

    // DBからデータを取得する
    function getPaymentDB($type){
        $objSPS =& LC_Mdl_SPS::getInstance();
		$objQuery =& SC_Query::getSingletonInstance();
		$arrVal = array($objSPS->getCode(true), (string)$type);
		$arrRet = $objQuery->select("module_id", "dtb_payment", "module_code = ? AND memo03 = ?", $arrVal);
        return $arrRet;
    }

    // onload設定
    function getBodyOnload($arrData) {
        if ($arrData['settlement_division'] == SETTLEMENT_API) {
            $tpl_onload = "lfCheckSettlement(". SETTLEMENT_API. ");";
        } else {
            $arrData['settlement_division'] = SETTLEMENT_LINK;
            $tpl_onload = "lfCheckSettlement(". SETTLEMENT_LINK. ");";
        }
        return $tpl_onload;
    }

    function lfIsRegistPaymentModule() {
        $objSPS =& LC_Mdl_SPS::getInstance();
        $module_code = $objSPS->getCode(true);
        $objQuery =& SC_Query_Ex::getSingletonInstance();
        if($objQuery->count('dtb_payment', 'module_code = ?', array($module_code))) {
            return true;
        }else{
            return false;
        }
    }


    /**
     * プラグイン情報をDB登録.
     *
     * @param string $plugin_code
     * @return void
     *
     */
    function lfRegistPlugin($plugin_code) {
        $objQuery =& SC_Query_Ex::getSingletonInstance();
        $sqlval = array();
		if($objQuery->count('dtb_plugin', 'plugin_code = ?', array($plugin_code))) {

            $sqlval['status'] = '1';
            $sqlval['enable'] = '1';
            $sqlval['del_flg'] = '0';
            $objQuery->update('dtb_plugin', $sqlval, 'plugin_code = ?', array($plugin_code));
			return;
		}

		$sqlval['plugin_id'] = $objQuery->get('max(plugin_id)', 'dtb_plugin') +1;
		$sqlval['plugin_name'] = $plugin_code;
        $sqlval['plugin_code'] = $plugin_code;
		$sqlval['class_name'] = $plugin_code;
        $sqlval['status'] = '1'; //2.11.0 unmatched flag PLUGIN_STATUS_UPLOADED;
        $sqlval['enable'] = '1'; //2.11.0 unmatched flag  PLUGIN_ENABLE_FALSE;
        $sqlval['del_flg'] = '0';
        $sqlval['update_date'] = 'now()';
        $objQuery->insert('dtb_plugin', $sqlval);
    }

}
?>
