<?php
/*
 * Copyright(c) 2000-2009 LOCKON CO.,LTD. All Rights Reserved.
 * http://www.lockon.co.jp/
 */

/**
 * 受注memo仕様
 * ・dtb_order, dtb_order_temp
 * memo01
 * memo02 コンビニ、ペイジーの入金通知結果(シリアライズされた値)
 * memo03 クレジット売上計上設定 売上日指定要求(null or 0)/自動売上(1)
 * memo04
 * memo05 リクエスト結果
 * memo06 トラッキングID
 * memo07
 * memo08
 * memo09
 * memo10
 */

// {{{ requires
require_once(MDL_SPS_CLASS_PATH. "LC_Mdl_SPS.php");
require_once(MDL_SPS_CLASS_PATH. "LC_Helper_Send_Payment.php");
require_once(MDL_SPS_CLASS_PATH. "util/LC_Util_SPS.php");
require_once(CLASS_PATH. "SC_CampaignSession.php");
require_once(CLASS_PATH . "pages/LC_Page.php");

class LC_Page_Mdl_SPS_Helper extends LC_Page {
    var $type;
    var $objQuery;
    var $objHelperDB;
    var $objFormParam;
    var $objSpsUtils;
    var $objSps;
    var $arrSetting;

    /**
     * コンストラクタ
     *
     * @return void
     */
    function LC_Page_Mdl_SPS_Helper($type) {
        $this->type = $type;
        $this->objQuery = new SC_Query();
        $this->objHelperDB = new SC_Helper_DB_Ex();
        $this->objFormParam = new SC_FormParam();
        $this->objSpsUtils = new LC_Util_SPS;
        $this->objSps =& LC_Mdl_SPS::getInstance();
        $this->arrSetting = $this->objSps->getUserSettings();
    }

    /**
     * Page を初期化する.
     *
     * @return void
     */
    function init() {
        parent::init();
        $this->tpl_column_num = 1;

        switch($this->type) {
        case SPS_CREDIT:
            if (defined("MOBILE_SITE") && MOBILE_SITE === true) {
                $this->tpl_mainpage = MDL_SPS_TEMPLATE_PATH. "api_card_mobile.tpl";
            } else {
                $this->tpl_mainpage = MDL_SPS_TEMPLATE_PATH. "api_card.tpl";
            }
            break;
        default:
            $objSPS->printLog("モジュールタイプエラー：". $this->type, PAYGENT_LOG_PATH);
            break;
        }
    }

    /**
     * Page のプロセス.
     *
     * @return void
     */
    function process() {
        $objView = (defined("MOBILE_SITE") && MOBILE_SITE === true) ? new SC_MobileView() : new SC_SiteView();
        $objSiteInfo = $objView->objSiteInfo;
        $this->arrInfo = $objSiteInfo->data;
        $this->objCampaignSess = new SC_CampaignSession();
        $this->objCartSess = new SC_CartSession();
        $this->objSiteSess = new SC_SiteSession();
        $this->objCustomer = new SC_Customer();

        // カート集計処理
        $this->objHelperDB->sfTotalCart($this, $this->objCartSess, $this->arrInfo);
        // ユーザユニークIDの取得と購入状態の正当性をチェック
        $this->uniqid = SC_Utils_Ex::sfCheckNormalAccess($this->objSiteSess, $this->objCartSess);
        // 一時受注テーブルの読込
        $arrData = $this->objHelperDB->sfGetOrderTemp($this->uniqid);
        // カート集計を元に最終計算
        $arrData = $this->objHelperDB->sfTotalConfirm($arrData, $this, $this->objCartSess, $this->arrInfo);
        // モード設定
        $mode = $this->getMode();
        // パラメータ情報の初期化
        $this->initParam($_POST);

        switch($mode) {
        // 前のページに戻る
        case 'return':
            // 正常な推移であることを記録しておく
            $this->objSiteSess->setRegistFlag();
            if (defined("MOBILE_SITE") && MOBILE_SITE === true) {
                header("Location: ". SC_Helper_Mobile::gfAddSessionId(MOBILE_URL_SHOP_CONFIRM));
            } else {
                header("Location: ". URL_SHOP_CONFIRM);
            }
            exit;
            break;
        // 次へ
        case 'next':
            $this->arrErr = $this->checkError($this->objFormParam->getHashArray());
            // 入力エラーなしの場合
            if(count($this->arrErr) == 0) {
                // 入力データの取得
                $arrInput = $this->objFormParam->getHashArray();
                switch($this->type) {
                // クレジット処理
                case SPS_CREDIT:
                    $this->exeSpsCredit($arrData, $arrInput);
                    break;
                }
            }
            break;
        default:
            break;
        }

        // 表示準備
        $this->dispData($_SESSION["payment_id"]);
        $this->arrForm = $this->objFormParam->getFormParamList();
        $objView->assignobj($this);
        $this->objCampaignSess->pageView($objView);
    }

    /* パラメータ情報の初期化 */
    function initParam($arrData = array()) {
        switch($this->type) {
        case SPS_CREDIT:
            $this->objFormParam->addParam("支払方法", "payment_class", INT_LEN, "n", array("EXIST_CHECK", "MAX_LENGTH_CHECK"));
            $this->objFormParam->addParam("分割回数", "divide_times", 3, "n", array());
            $this->objFormParam->addParam("ボーナス併用回数", "bonus_details_times", 1, "n", array());
            if ($_POST['stock_card'] != 1) {
                $this->objFormParam->addParam("カード番号1", "card_no01", CREDIT_NO_LEN, "n", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK"));
                $this->objFormParam->addParam("カード番号2", "card_no02", CREDIT_NO_LEN, "n", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK"));
                $this->objFormParam->addParam("カード番号3", "card_no03", CREDIT_NO_LEN, "n", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK"));
                $this->objFormParam->addParam("カード番号4", "card_no04", CREDIT_NO_LEN, "n", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK"));
                $this->objFormParam->addParam("カード期限年", "card_year", 4, "n", array("EXIST_CHECK", "NUM_COUNT_CHECK", "NUM_CHECK"));
                $this->objFormParam->addParam("カード期限月", "card_month", 2, "n", array("EXIST_CHECK", "NUM_COUNT_CHECK", "NUM_CHECK"));
                //$this->objFormParam->addParam("姓", "card_name01", 32, "KVa", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "ALPHA_CHECK"));
                //$this->objFormParam->addParam("名", "card_name02", 32, "KVa", array("EXIST_CHECK", "MAX_LENGTH_CHECK", "ALPHA_CHECK"));
                $this->objFormParam->addParam("カード情報保持", "stock", 1, "n", array("NUM_COUNT_CHECK", "NUM_CHECK"));
                break;
            }
        }

        $this->objFormParam->setParam($arrData);
        $this->objFormParam->convParam();
   }

    /* 入力内容のチェック */
    function checkError($arrData) {
        // 共通エラーチェック
        $arrErr = $this->objFormParam->checkError();

        // 個別エラーチェック
        $objErr = new SC_CheckError($arrData);
        switch($this->type) {
        case SPS_CREDIT:
            if ($arrData['payment_class'] == SPS_CREDIT_PAYMENT_TYPE_BB || $arrData['payment_class'] == SPS_CREDIT_PAYMENT_TYPE_D) {
                $objErr->doFunc(array("分割回数", "divide_times", 3), array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK", "ZERO_CHECK"));
            }
            if ($arrData['payment_class'] == SPS_CREDIT_PAYMENT_TYPE_BB) {
                $objErr->doFunc(array("ボーナス併用回数", "bonus_details_times", 1), array("EXIST_CHECK", "MAX_LENGTH_CHECK", "NUM_CHECK", "ZERO_CHECK"));
                $objErr->doFunc(array("ボーナス併用回数", "bonus_details_times", 6), array("MAX_CHECK"));
            }
            break;
        }

        $arrErr = array_merge((array)$arrErr, (array)$objErr->arrErr);
        return $arrErr;
    }

    /**
     * モード設定
     */
    function getMode() {
        $mode = '';
        if (isset($_POST['mode'])) {
            $mode = $_POST['mode'];
        }
        return $mode;
    }

    /**
     * 表示処理
     *
     * @param int $payment_id
     */
    function dispData($payment_id) {
        // 支払方法の説明画像を取得
        $arrRet = $this->objQuery->select("payment_method, payment_image", "dtb_payment", "payment_id = ?", array($payment_id));
        $this->tpl_title = $arrRet[0]['payment_method'];
        $this->tpl_payment_method = $arrRet[0]['payment_method'];
        $this->tpl_payment_image = $arrRet[0]['payment_image'];

        // その他
        switch($this->type) {
        case SPS_CREDIT:
            // クレジット支払回数
            if (defined('SPS_CREDIT_PAYMENT_TYPE_L')) $this->arrPaymentClass[SPS_CREDIT_PAYMENT_TYPE_L] = "一括";
            if (defined('SPS_CREDIT_PAYMENT_TYPE_BL')) $this->arrPaymentClass[SPS_CREDIT_PAYMENT_TYPE_BL] = "ボーナス一括";
            if (defined('SPS_CREDIT_PAYMENT_TYPE_BB')) $this->arrPaymentClass[SPS_CREDIT_PAYMENT_TYPE_BB] = "ボーナス併用";
            if (defined('SPS_CREDIT_PAYMENT_TYPE_D')) $this->arrPaymentClass[SPS_CREDIT_PAYMENT_TYPE_D] = "分割";
            if (defined('SPS_CREDIT_PAYMENT_TYPE_R')) $this->arrPaymentClass[SPS_CREDIT_PAYMENT_TYPE_R] = "リボ";
            // 月日配列
            $objDate = new SC_Date();
            $objDate->setStartYear(date('Y'));
            $objDate->setEndYear(date('Y') + CREDIT_ADD_YEAR);
            $this->arrYear = $objDate->getYear();
            $this->arrMonth = $objDate->getZeroMonth();
            break;
        }
    }

    /**
     * クレジット処理
     *
     * @param array $arrData
     * @param array $arrInput
     */
    function exeSpsCredit($arrData, $arrInput) {
        // 売上計上設定を一時受注データに保持
        $this->objHelperDB->sfRegistTempOrder($this->uniqid, array("memo03" => $this->arrSetting['type_sps_auth']));
        // XML用にデータを配列化
        $arrXml = array("merchant_id" => $this->arrSetting['id_sps_marchant'],
                        "service_id" => $this->arrSetting['id_sps_service'],
                        "cust_code" => ($arrData['customer_id'] != 0) ? $arrData['customer_id'] : NON_MEMBER_PREFIX. md5(uniqid(rand(), true)),
                        "order_id" => $arrData['order_id']. date('YmdHis'),
                        "item_id" => "ec-cube",
                        "amount" => $arrData['payment_total'],
                        "pagecon_url" => MDL_SPS_SETTLEMENT_URL,
                        "free1" => $arrData['order_id'],
                        "pay_method_info" => array("cc_number" => $arrInput['card_no01']. $arrInput['card_no02']. $arrInput['card_no03']. $arrInput['card_no04'],
                                                   "cc_expiration" => $arrInput['card_year']. $arrInput['card_month'],
                                                   "dealings_type" => $arrInput['payment_class'],
                                                   "divide_times" => ($arrInput['divide_times'] > 0) ? sprintf("%03d", $arrInput['divide_times']) : "",
                                                   "bonus_details_times" => $arrInput['bonus_details_times'],
                                                   "cust_manage_flg" => sprintf("%01d", $arrInput['stock'])),
                        "encrypted_flg" => 1,
                        "request_date" => date('YmdHis'),
                        "limit_second" => 600);
        // ハッシュ生成
        $arrXml['sps_hashcode'] = $this->objSpsUtils->createHashcode($arrXml, $this->arrSetting);
        // 固有情報を暗号化
        $arrXml = $this->objSpsUtils->encryptData($arrXml, "pay_method_info", $this->arrSetting);
        // マルチバイト文字列のエンコード
        $arrXml = $this->objSpsUtils->encodeMbstring($arrXml, array("free1"));
        // XML生成
        $requestXml = $this->objSpsUtils->changeArray2Xml($arrXml, SPS_CREDIT_AUTH_CODE);
        // リクエスト
        list($r_code, $response) = $this->objSpsUtils->sendRequestSps($requestXml, $this->arrSetting);
        // 確定処理
        $this->exeSpsCreditCommit($r_code, $response, $arrData['payment_total']);
    }

    /**
     * 確定処理
     *
     * @param int $r_code
     * @param xml $response
     * @param int $payment_total
     */
    function exeSpsCreditCommit($r_code, $response, $payment_total) {
        // レスチェック
        $arrRes = $this->checkRespoinse($r_code, $response);
        if ($arrRes === false) {
            return;
        }

        // トラッキングIDを一時受注データに保持
        $this->objHelperDB->sfRegistTempOrder($this->uniqid, array("memo06" => $arrRes['res_tracking_id']));
        // XML用にデータを配列化
        $arrXml = array("merchant_id" => $this->arrSetting['id_sps_marchant'],
                        "service_id" => $this->arrSetting['id_sps_service'],
                        "sps_transaction_id" => $arrRes['res_sps_transaction_id'],
                        "tracking_id" => $arrRes['res_tracking_id'],
                        "processing_datetime" => $arrRes['res_process_date'],
                        "request_date" => date('YmdHis'),
                        "limit_second" => 600);
        // ハッシュ生成
        $arrXml['sps_hashcode'] = $this->objSpsUtils->createHashcode($arrXml, $this->arrSetting);
        // XML生成
        $requestXml = $this->objSpsUtils->changeArray2Xml($arrXml, SPS_CREDIT_COMMIT_CODE);
        // リクエスト
        list($r_code, $response) = $this->objSpsUtils->sendRequestSps($requestXml, $this->arrSetting);
        // 完了処理
        $this->exeFix($r_code, $response, $payment_total);
    }

    /**
     * 完了処理
     *
     * @param int $r_code
     * @param xml $response
     * @param int $payment_total
     */
    function exeFix($r_code, $response, $payment_total) {
        // レスチェック
        $arrRes = $this->checkRespoinse($r_code, $response);
        if ($arrRes === false) {
            return;
        }

        // 正常に登録されたことを記録
        $this->objSiteSess->setRegistFlag();
        LC_Helper_Send_Payment::sendPaymentData(MDL_SPS_CODE, $payment_total);
        // 完了画面に遷移
        if (defined("MOBILE_SITE") && MOBILE_SITE === true) {
            header("Location: ". SC_Helper_Mobile_Ex::gfAddSessionId(MOBILE_URL_SHOP_COMPLETE));
        } else {
            header("Location: ". URL_SHOP_COMPLETE);
        }
    }

    /**
     * レスチェック処理
     *
     * @param int $r_code
     * @param xml $response
     */
    function checkRespoinse($r_code, $response) {
        // 通信エラー
        if ($r_code != 200) {
            $this->tpl_error = "SBPSとの通信に失敗しました。";
            return false;
        // レス処理
        } else {
            // XMLを配列化
            $arrRes = $this->objSpsUtils->changeXml2Array($response);
            // 成功
            if($arrRes['res_result'] == "OK") {
                return $arrRes;
            // 失敗
            } else {
                $this->tpl_error = "決済に失敗しました。";
                $this->tpl_error .= (strlen($arrRes['res_err_code']) > 0) ? "（エラーコード：${arrRes['res_err_code']}）" : "";
                return false;
            }
        }
    }
}
?>
