<?php
/*
 * Copyright(c) 2011-2012 SOFTBANK PAYMENT SERVICE Corp. All rights reserved.
 * http://www.sbpayment.jp/
 * Copyright(c) 2000-2009 LOCKON CO.,LTD. All Rights Reserved.
 * http://www.lockon.co.jp/
 */

require_once(DATA_REALDIR . 'module/Request.php');
require_once(MDL_SPS_CLASS_PATH. "LC_Mdl_SPS.php");
require_once(MDL_SPS_MODULE_PATH. 'XML/Serializer.php');
require_once(MDL_SPS_MODULE_PATH. 'XML/Unserializer.php');

/**
 * SPS用各種ユーティリティクラス
 */
class LC_Util_SPS {
    /**
     * 禁止文字か判定を行う。
     *
     * @param string $value 判定対象
     * @return boolean 結果
     */
    function isNgChar($char) {
        if ($char == '') return false;
        // unicode base
        if (bin2hex($char) == 'e28094') return true; // ダッシュ亜種
        if (bin2hex($char) == 'e28095') return true; // ダッシュ
        if (bin2hex($char) == '5c') return true; // バックスラッシュ　＝　￥マーク
        if (bin2hex($char) == 'e3809c') return true; // 波ダッシュ
        if (bin2hex($char) == 'e28096') return true; // パラレル
        if (bin2hex($char) == 'e28892') return true; // 二分ダッシュ
        if (bin2hex($char) == 'c2a2') return true; // セント
        if (bin2hex($char) == 'c2a3') return true; // ポンド
        if (bin2hex($char) == 'c2ac') return true; // 否定算術記号

        $check_char = mb_convert_encoding($char, "SJIS-win", "UTF-8");
        // convert sjis base
        if ($check_char == '') return true;
        if (bin2hex($check_char) == '815d') return true;
        if (bin2hex($check_char) == '815f') return true;
        if (bin2hex($check_char) == '8160') return true;
        if (bin2hex($check_char) == '8161') return true;
        if (bin2hex($check_char) == '817c') return true;
        if (bin2hex($check_char) == '8191') return true;
        if (bin2hex($check_char) == '8192') return true;
        if (bin2hex($check_char) == '81ca') return true;
        return false;
    }

    /**
     * NG文字を置換する。
     *
     * @param string $str 対象文字列
     * @return string 結果
     */
    function convNgChar($str) {
        for ($i = 0; $i < mb_strlen($str); $i++) {
            $tmp = mb_substr($str, $i , 1);
            if (LC_Util_SPS::isNgChar($tmp)) {
                $ret .= SPS_REPLACE_NG_CHAR;
            } else {
                $ret .= $tmp;
            }
        }
        return $ret;
    }

    /**
     * SPSへリクエストを送信
     * @param string $requestXml
     * @param array $spsSetting
     * @return string $r_code
     * @return string $response
     */
    function sendRequestSps($requestXml, $spsSetting) {
        $objSPS =& LC_Mdl_SPS::getInstance();
        $objSPS->printLog($requestXml);
        $order_url = $objSPS->getUserSettings( 'url_sps_api' );
        // リクエスト処理
        $req = new HTTP_Request($order_url);
        $req->addHeader('Content-Type', 'text/xml');
        $req->setMethod('POST');
        $req->setBasicAuth($spsSetting['id_sps_marchant']. $spsSetting['id_sps_service'], $spsSetting['id_sps_hash']);
        $req->setBody($requestXml);
        $req->sendRequest();
        $r_code = $req->getResponseCode();
        $response = $req->getResponseBody();
        // レスポンスをログに保持
        $objSPS =& LC_Mdl_SPS::getInstance();
        $objSPS->printLog($response);

        return array($r_code, $response);
    }

    /**
     * 配列をXML化
     * @param array $arrXml
     * @param string $request_code
     * @return string $xml
     */
    function changeArray2Xml($arrXml, $request_code) {
        // 配列をXML化
        $objXmlSerializer = new XML_Serializer();
        $objXmlSerializer->setOption("indent", "    ");
        $objXmlSerializer->setOption("addDecl", true);
        $objXmlSerializer->setOption("encoding", "Shift_JIS");
        $objXmlSerializer->setOption("rootName", "sps-api-request");
        $objXmlSerializer->setOption("rootAttributes", array("id" => $request_code));
        $objXmlSerializer->serialize($arrXml);
        $xml = $objXmlSerializer->getSerializedData();

        return $xml;
    }

    /**
     * XMLを配列化
     * @param string $xml
     * @return array $arrXml
     */
    function changeXml2Array($xml) {
        // 不要なデータを排除
        $xml = preg_replace("/<\?xml.*\?>/", "", $xml);

        // XMLを配列化
        $objXmlUnserializer = new XML_Unserializer();
        $objXmlUnserializer->setOption("parseAttributes", true);
        $objXmlUnserializer->unserialize($xml);
        $arrXml = $objXmlUnserializer->getUnserializedData();

        return $arrXml;
    }

    /**
     * ハッシュ生成
     * @param array $arrData
     * @param array $spsSetting
     * @return string $hashcode
     */
    function createHashcode($arrData, $spsSetting) {
        foreach ($arrData as $key1 => $val1) {
            if (is_array($val1) === false) {
                $hashcode .= trim($val1);
            } else {
                foreach ($val1 as $key2 => $val2) {
                    $hashcode .= trim($val2);
                }
            }
        }
        $hashcode = sha1($hashcode. trim($spsSetting['id_sps_hash']));

        return $hashcode;
    }

    /**
     * 暗号化
     * @param array $arrData
     * @param string $pkey
     * @param array $spsSetting
     * @return array $arrData
     */
    function encryptData($arrData, $pkey, $spsSetting) {
        // パディング（最後の8バイトブロックに半角スペースを補完）
        foreach ($arrData[$pkey] as $key => $val) {
            $val_len = strlen($val);
            if ($val_len > 0) {
                $val_last_8byte = $val_len % 8;
                $val_null_8byte = ($val_last_8byte != 0) ? 8 - $val_last_8byte : 0;
                for ($i = 0; $i < $val_null_8byte; $i++) {
                    $arrData[$pkey][$key] .= " ";
                }
            }
        }

        // 暗号化
        $em = mcrypt_module_open(MCRYPT_3DES, "",  MCRYPT_MODE_CBC, "");
        foreach ($arrData[$pkey] as $key => $val) {
            if (strlen($val) > 0) {
                mcrypt_generic_init($em, $spsSetting['key_sps_3des'], $spsSetting['key_sps_3des_iv']);
                $encrypt_val = mcrypt_generic($em, $val);
                mcrypt_generic_deinit($em);
                $arrData[$pkey][$key] = base64_encode($encrypt_val);
            }
        }
        mcrypt_module_close($em);

        return $arrData;
    }

    /**
     * マルチバイト文字列のエンコード
     * @param array $arrData
     * @param array $key
     * @return array $arrData
     */
    function encodeMbstring($arrData, $arrKey) {
        foreach ($arrKey as $key) {
            $arrData[$key] = base64_encode(LC_Util_SPS::convNgChar($arrData[$key]));
        }

        return $arrData;
    }
}
?>
