<?php
/*
 * Copyright(c) 2011 SOFTBANK PAYMENT SERVICE Corp. All rights reserved.
 * http://www.sbpayment.jp/
 */

require_once('../require.php');
require_once(MODULE_REALDIR . 'mdl_sps/inc/include.php');
require_once(MODULE_REALDIR . "mdl_sps/class/LC_Helper_Send_Payment.php");

require_once(CLASS_EX_REALDIR . "helper_extends/SC_Helper_Purchase_Ex.php");

/**
 * 受注memo仕様
 * ・dtb_order, dtb_order_temp
 * memo01
 * memo02 コンビニ、ペイジーの入金通知結果(シリアライズされた値)
 * memo03 クレジット売上計上設定 売上日指定要求(null or 0)/自動売上(1)
 * memo04
 * memo05 リクエスト結果
 * memo06 トラッキングID
 * memo07
 * memo08
 * memo09
 * memo10
 */
function gfPrintLog($msg, $log_path) {
	GC_Utils::gfPrintLog($msg, $log_path);
}

$objView = new SC_AdminView();
$objQuery = new SC_Query();
$objSiteSess = new SC_SiteSession();
$objCartSess = new SC_CartSession();
$objCustomer = new SC_Customer();
$objDb = new SC_Helper_DB_Ex();
$objMail = new SC_Helper_Mail();

$objSiteInfo = $objView->objSiteInfo;
$arrInfo = $objSiteInfo->data;

$log_path = DATA_REALDIR . "logs/sps.log";
gfPrintLog("**************************************** sps start ****************************************", $log_path);

// POSTの内容を全てログ保存
$arrResult = $_POST;
foreach($arrResult as $key => $val){
    gfPrintLog( "\t" . $key . " => " . $val, $log_path);
}

// order_idの入れ替え（order_idのみを抽出：order_id＋YYYYMMDDHHMISS への対応）
$order_len = mb_strlen($arrResult['free1']) - 14 ;
if($order_len > 0){
    $arrResult['order_id'] = substr($arrResult['free1'],0,$order_len);
}else{
    $arrResult['order_id'] = $arrResult['free1'];
}

switch ($arrResult["res_result"]) {
case 'OK':
    $objQuery->begin();
    lfDoComplete($objQuery, $arrResult, $log_path, $objDb);
	$order_id = $arrResult['order_id'];
    $objQuery->commit();
    break;
case 'PY':
    gfPrintLog('---> SPS PY Start', $log_path);
    SPSNotify($arrResult, $log_path);
    gfPrintLog('---> SPS PY End', $log_path);
    exit;
case 'CN':
    gfPrintLog('---> SPS CN Start', $log_path);
    SPSNotifyCancel($arrResult, $log_path);
    gfPrintLog('---> SPS CN End', $log_path);
    exit;
default:
    echo "NG";
    gfPrintLog('---> SPS NG Start', $log_path);
    SPSNotifyCancel($arrResult, $log_path);
    gfPrintLog('---> SPS NG End', $log_path);
    exit;
}


// 完了メール送信
if(SC_Utils::sfIsInt($order_id) && $order_id > 0) {
    $arrRet = $objQuery->select("order_email, memo03, payment_id", "dtb_order", "order_id = ?", array($order_id));
    //登録されているメールアドレスが携帯かPCかに応じて注文完了メールのテンプレートを変える
    if(preg_match("/(ezweb.ne.jp$|docomo.ne.jp$|softbank.ne.jp$|vodafone.ne.jp$)/", $arrRet[0]['order_email'])){
        // モバイル版
        $objMail->sfSendOrderMail($order_id, '2');
    }else{
        // PC版
        $objMail->sfSendOrderMail($order_id, '1');
    }
    /**
     * ・支払い方法がクレジットかつ自動売上設定の場合
     * ・支払い方法がウェブマネーの場合
     * は、対応状況を入金済みに変更
     */
    $payMethod = $objQuery->get('memo03', 'dtb_payment', 'payment_id = ?', array($arrRet[0]['payment_id']));
    if ((($payMethod == SPS_CREDIT || $payMethod == SPS_CREDIT3D) && $arrRet[0]['memo03'] == SPS_AUTH_AUTO) || $payMethod == SPS_WEBMONEY || $payMethod == SPS_YWALLET) {
        $objQuery->update('dtb_order', array("status" => ORDER_PRE_END), 'order_id = ?', array($order_id));
    }
    print("OK");

// エラーの場合受信データを送信
} else {
    gfPrintLog("!!!!!!!!!!!! sps error !!!!!!!!!!!!!!", $log_path);
	print("NG");
}

gfPrintLog("**************************************** sps end ****************************************", $log_path);

// 完了処理
function lfDoComplete($objQuery, $arrResult, $log_path, $objDb) {

    $arrCrilentIP = $objQuery->select("memo01, memo02, memo03, memo04, memo05", "dtb_payment", "module_code = ? and del_flg = 0", array(MDL_SPS_CODE));

    // レスポンスデータがない場合にはエラー
    if(count($arrResult) <= 0) {
        gfPrintLog("決済サーバからデータを正しく受け取れませんでした。", $log_path);
        return;
    }

    // マーチャントIDが異なる場合はエラー
	if($arrResult["merchant_id"] != $arrCrilentIP[0]['memo01']) {
        gfPrintLog("マーチャントIDが違います。", $log_path);
        return;
    }

    // サービスIDが異なる場合はエラー
    if($arrResult["service_id"] != $arrCrilentIP[0]['memo02']) {
        gfPrintLog("サービスIDが違います。", $log_path);
        return;
    }

    $objPurchase = new SC_Helper_Purchase_Ex();
    $sqlval = array();
    $sqlval['memo06'] = $arrResult['res_tracking_id'];
    $objPurchase->sfUpdateOrderStatus($arrResult['order_id'], ORDER_PAY_WAIT, null, null, $sqlval);

	$arrData = $objQuery->select('*', 'dtb_order', 'order_id = ?', array($arrResult['order_id']));
	$arrData = $arrData[0];

    // 正常な売上であることを登録する。
    LC_Helper_Send_Payment::sendPaymentData(MDL_SPS_CODE, $arrData["payment_total"]);
    return $order_id;
}


function SPSNotify($arrResult, $log_path) {
    $objForm = new SC_FormParam;
    $objForm->addParam('res_payinfo_key', 'res_payinfo_key', 32, '', array('EXIST_CHECK'));
    $objForm->addParam('res_tracking_id', 'res_tracking_id', 14, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->addParam('order_id', 'order_id', 38, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->addParam('item_id', 'item_id', 32, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->setParam($arrResult);

    if ($arrErr = $objForm->checkError()) {
        gfPrintLog('パラメータが不正です。', $log_path);
        gfPrintLog(print_r($arrErr, true), $log_path);
        print "NG";
        return;
    }

    $objQuery = new SC_Query();
    $arrRet = $objQuery->select('*', 'dtb_order', 'order_id = ?',
        array($objForm->getValue('order_id')));

    if (empty($arrRet)) {
        gfPrintLog('受注が存在しません。オーダーID -> ' . $objForm->getValue('order_id'), $log_path);
        print "NG";
        return;
    }

    $pkey = $objForm->getValue('res_payinfo_key');
    list($type, $pay, $total, $email) = split(',', $pkey);

    switch($type) {
    case 'P':
    case 'D':
    case 'C':
    case 'G':
        $arrMemo02 = array(
            'title' => array(
                'name'  => '',
                'value' => true
            ),
            'pay' => array(
                'name'  => '入金額',
                'value' => $pay
            ),
            'total' => array(
                'name'  => '入金累計額',
                'value' => $total
            ),
            'email' => array(
                'name'  => 'メールアドレス',
                'value' => $email
            ),
        );

        $sqlVal = array(
            'memo02' => serialize($arrMemo02),
            'status' => ORDER_PRE_END,
            'update_date' => 'NOW()',
        );
        if($type == 'C') {
            $sqlVal['status'] = ORDER_PAY_WAIT;
        }
        $objQuery->update(
            'dtb_order',
            $sqlVal,
            'order_id = ?',
            array($objForm->getValue('order_id'))
        );
        break;

    default:
        gfPrintLog('通知種別が不正です。種別 -> ' . $type, $log_path);
        print "NG";
        return;
    }

    print "OK";
}

function SPSNotifyCancel($arrResult, $log_path) {
    $objForm = new SC_FormParam;
    $objForm->addParam('res_tracking_id', 'res_tracking_id', 14, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->addParam('order_id', 'order_id', 38, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->addParam('item_id', 'item_id', 32, '', array('EXIST_CHECK', 'MAX_LENGTH_CHECK'));
    $objForm->setParam($arrResult);

    if ($arrErr = $objForm->checkError()) {
        gfPrintLog('パラメータが不正です。', $log_path);
        gfPrintLog(print_r($arrErr, true), $log_path);
        print "NG";
        return;
    }

    $objQuery = new SC_Query();
    $arrRet = $objQuery->select('*', 'dtb_order', 'order_id = ?',
        array($objForm->getValue('order_id')));

    if (empty($arrRet)) {
        gfPrintLog('受注が存在しません。オーダーID -> ' . $objForm->getValue('order_id'), $log_path);
        print "NG";
        return;
    }

    $objQuery->update(
        'dtb_order', array('status' => ORDER_CANCEL, 'update_date' => 'NOW()'),
        'order_id = ?', array($objForm->getValue('order_id'))
    );
    print "OK";
}
?>
