<?php
/*
 * Copyright(c) 2011 SOFTBANK PAYMENT SERVICE Corp. All rights reserved.
 * http://www.sbpayment.jp/
 * Copyright(c) 2000-2009 LOCKON CO.,LTD. All Rights Reserved.
 * http://www.lockon.co.jp/
 */
require_once(MODULE_REALDIR . 'mdl_sps/inc/include.php');
require_once(MDL_SPS_CLASS_PATH. 'LC_Mdl_SPS.php');
require_once(MDL_SPS_CLASS_PATH. "util/LC_Util_SPS.php");


define('MODULE_PATH', MODULE_REALDIR);
/**
 * 受注memo仕様
 * ・dtb_order, dtb_order_temp
 * memo01
 * memo02 コンビニ、ペイジーの入金通知結果(シリアライズされた値)
 * memo03 クレジット売上計上設定 売上日指定要求(null or 0)/自動売上(1)
 * memo04
 * memo05 リクエスト結果
 * memo06 トラッキングID
 * memo07
 * memo08
 * memo09
 * memo10
 */

/*
 * SPSへリクエストを送信し結果を取得
 * @param 	int 	$order_id 		受注ID
 * @param 	string 	$request_type 	リクエスト種別[commit or cancel]
 * @return	bool
 */
function sfSpsRequest($order_id, $request_type) {
	// DBから受注情報を取得
    $objQuery =& SC_Query_Ex::getSingletonInstance();
	$objDb = new SC_Helper_DB_Ex();
	$objSPS =& LC_Mdl_SPS::getInstance();

	$where = "order_id = ?";
	$tmpOrder = $objQuery->select( "*", "dtb_order", $where, array( $order_id ) );
	$arrOrder = $tmpOrder[0];

	$proccess_datetime = $_POST['sps_year']
	                   . sprintf('%02d', $_POST['sps_month'])
	                   . sprintf('%02d', $_POST['sps_date'])
	                   . '000000';
	// 返金処理の場合は、過去日付が入力された場合エラーとする.
    $order_datetime = str_replace('-', '', substr($arrOrder['create_date'], 0, 10)) . '000000';
    if ($request_type == SPS_REQUEST_TYPE_CANCEL && $proccess_datetime < $order_datetime) {
        $objSPS->printLog('debug: request datetime check');
        $objSPS->printLog("order_datetime: $order_datetime");
        $objSPS->printLog("proccess_datetime: $proccess_datetime");
        return '受注日より以前の日付は指定できません。';
    }

	// リクエスト用 xml受注配列を取得
	$requestXml = sfCreateXml($arrOrder, $request_type, $proccess_datetime);

	// SPSへリクエストを送信
	list($r_code, $response) = LC_Util_SPS::sendRequestSps($requestXml, $objSPS->getUserSettings());
	if ($r_code != 200) {	// NG
		// dtb_order memo05 を 「３：通信失敗」に更新
		$sqlval['memo05'] = SPS_ERROR;
		$res = "SBPSよりエラーが返されました。";
	} else {
		// XMLを配列化
		$arrRes = LC_Util_SPS::changeXml2Array($response);
        if(PEAR::isError($arrRes)) {
            $sqlval['memo05'] = SPS_ERROR;
            $res = mb_convert_encoding($response, 'UTF-8', 'SJIS');
        } else if ($arrRes['res_result'] == "NG") {
            $sqlval['memo05'] = SPS_ERROR;
            $res = "SBPSよりエラーが返されました。\\nエラーコード：". $arrRes['res_err_code'];
	    } else {
    	    // OK
    		switch ($request_type) {
    			case SPS_REQUEST_TYPE_COMMIT:
    				// dtb_order memo05 を 「１：確定」に更新
    				$sqlval['memo05'] = SPS_COMMIT_OK;
    				$res = "確定要求が正常に完了しました。";
    				break;
                case SPS_REQUEST_TYPE_DATECOMMIT:
                    // dtb_order memo05 を 「4：売上日指定確定」に更新
                    $sqlval['memo05'] = SPS_DATECOMMIT_OK;
                    $sqlval['status'] = ORDER_PRE_END;
                    $res = "売上日指定要求が正常に完了しました。";
                    break;
    			case SPS_REQUEST_TYPE_CANCEL:
    				// dtb_order memo05 を 「２：取消（返金）」に更新
    				$sqlval['memo05'] = SPS_CANCEL_OK;
    				$sqlval['status'] = ORDER_CANCEL;
    				$res = "返金要求が正常に完了しました。";
    				break;
    		}
		}
	}

	$objQuery->begin();
	$objQuery->update( "dtb_order", $sqlval, "order_id = ?", array( $order_id ) );
	$objQuery->commit();

	return $res;
}

/*
 * 受注配列からxmlを生成
 * @param 	array 	$arrOrder 		受注情報
 * @param 	strint 	$request_type 	リクエスト種別[commit or cancel]
 * @return	string	$requestXml		xml受注情報
 */
function sfCreateXml($arrOrder, $request_type , $proccess_datetime) {
	$objSPS =& LC_Mdl_SPS::getInstance();
	$spsSetting = $objSPS->getUserSettings();

	// リクエストコード
	switch ($request_type) {
	case SPS_REQUEST_TYPE_COMMIT:
		$request_code = SPS_CREDIT_COMMIT_CODE;
		break;
	case SPS_REQUEST_TYPE_DATECOMMIT:
		$request_code = SPS_CREDIT_DATECOMMIT_CODE;
		break;
	case SPS_REQUEST_TYPE_CANCEL:
		// クレジット
		if ($_POST['payment_type'] == SPS_CREDIT) {
			$request_code = SPS_CREDIT_CANCEL_CODE;
        } elseif ($_POST['payment_type'] == SPS_CREDIT3D) {
            $request_code = SPS_CREDIT_CANCEL_CODE;
		// Yahoo!ウォレット
		} elseif ($_POST['payment_type'] == SPS_YWALLET) {
			$request_code = SPS_YWALLET_CANCEL_CODE;
		}
		break;
	}
	// リクエスト日時
	$requestDate = date('YmdHis');
	// ハッシュコード
    $hashCode = sha1(
                trim($spsSetting['id_sps_marchant'])
              . trim($spsSetting['id_sps_service'])
              . trim($arrOrder['memo06']) // tracking_id
              . trim($proccess_datetime)
              . trim($requestDate)
              . "600"
              . trim($spsSetting['id_sps_hash'])
    );

    // XML用配列
    $arrXml = array("merchant_id" => $spsSetting['id_sps_marchant'],
                    "service_id" => $spsSetting['id_sps_service'],
                    "tracking_id" => $arrOrder['memo06'],
                    "processing_datetime" => $proccess_datetime,
                    "request_date" => $requestDate,
                    "limit_second" => 600,
                    "sps_hashcode" => $hashCode);
    // 配列をXML化
    $requestXml = LC_Util_SPS::changeArray2Xml($arrXml, $request_code);

    return $requestXml;
}
